#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <pthread.h>
#include <semaphore.h>
#include <string.h>
#include <setjmp.h>
#include <signal.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/wait.h>

typedef void handler_t(int);

/* 
 * macro for posix-style error handling
 */
#define posix_error(code,msg) do {\
  printf("%s: %s\n", msg, strerror(code));\
  exit(0);\
} while (0)

/*
 * macro for unix-style error handling 
 */
#define unix_error(msg) do {\
  printf("%s: %s\n", msg, strerror(errno));\
  exit(0);\
} while (0)

/* process control functions */
pid_t Fork(void);
pid_t Wait(int *status);
handler_t *Signal(int signum, handler_t *handler);
void Kill(pid_t pid, int signum);
unsigned int Sleep(unsigned int secs);
void Pause(void);

/* thread control functions */
void Pthread_create(pthread_t *tidp, pthread_attr_t *attrp, 
		    void * (*routine)(void *), void *argp);
void Pthread_join(pthread_t tid, void **thread_return);
void Pthread_cancel(pthread_t tid);

/* semaphore functions */
void Sem_init(sem_t *sem, int pshared, unsigned int value);
void P(sem_t *sem);
void V(sem_t *sem);

/* mutex functions */
void Pthread_mutex_init(pthread_mutex_t *mutex, pthread_mutexattr_t *attr);
void Pthread_mutex_lock(pthread_mutex_t *mutex);
void Pthread_mutex_unlock(pthread_mutex_t *mutex);

/* condition variable functions */
void Pthread_cond_init(pthread_cond_t *cond, pthread_condattr_t *attr);
void Pthread_cond_signal(pthread_cond_t *cond);
void Pthread_cond_broadcast(pthread_cond_t *cond);
void Pthread_cond_wait(pthread_cond_t *cond, pthread_mutex_t *mutex);
int Pthread_cond_timedwait(pthread_cond_t *cond, pthread_mutex_t *mutex,
			    struct timespec *abstime);
